@@ -140,110 +140,112 @@
 #define KEY_opendir		139
 #define KEY_or			140
 #define KEY_ord			141
-#define KEY_pack		142
-#define KEY_package		143
-#define KEY_pipe		144
-#define KEY_pop			145
-#define KEY_pos			146
-#define KEY_print		147
-#define KEY_printf		148
-#define KEY_prototype		149
-#define KEY_push		150
-#define KEY_q			151
-#define KEY_qq			152
-#define KEY_quotemeta		153
-#define KEY_qw			154
-#define KEY_qx			155
-#define KEY_rand		156
-#define KEY_read		157
-#define KEY_readdir		158
-#define KEY_readline		159
-#define KEY_readlink		160
-#define KEY_readpipe		161
-#define KEY_recv		162
-#define KEY_redo		163
-#define KEY_ref			164
-#define KEY_rename		165
-#define KEY_require		166
-#define KEY_reset		167
-#define KEY_return		168
-#define KEY_reverse		169
-#define KEY_rewinddir		170
-#define KEY_rindex		171
-#define KEY_rmdir		172
-#define KEY_s			173
-#define KEY_scalar		174
-#define KEY_seek		175
-#define KEY_seekdir		176
-#define KEY_select		177
-#define KEY_semctl		178
-#define KEY_semget		179
-#define KEY_semop		180
-#define KEY_send		181
-#define KEY_setgrent		182
-#define KEY_sethostent		183
-#define KEY_setnetent		184
-#define KEY_setpgrp		185
-#define KEY_setpriority		186
-#define KEY_setprotoent		187
-#define KEY_setpwent		188
-#define KEY_setservent		189
-#define KEY_setsockopt		190
-#define KEY_shift		191
-#define KEY_shmctl		192
-#define KEY_shmget		193
-#define KEY_shmread		194
-#define KEY_shmwrite		195
-#define KEY_shutdown		196
-#define KEY_sin			197
-#define KEY_sleep		198
-#define KEY_socket		199
-#define KEY_socketpair		200
-#define KEY_sort		201
-#define KEY_splice		202
-#define KEY_split		203
-#define KEY_sprintf		204
-#define KEY_sqrt		205
-#define KEY_srand		206
-#define KEY_stat		207
-#define KEY_study		208
-#define KEY_sub			209
-#define KEY_substr		210
-#define KEY_symlink		211
-#define KEY_syscall		212
-#define KEY_sysopen		213
-#define KEY_sysread		214
-#define KEY_sysseek		215
-#define KEY_system		216
-#define KEY_syswrite		217
-#define KEY_tell		218
-#define KEY_telldir		219
-#define KEY_tie			220
-#define KEY_tied		221
-#define KEY_time		222
-#define KEY_times		223
-#define KEY_tr			224
-#define KEY_truncate		225
-#define KEY_uc			226
-#define KEY_ucfirst		227
-#define KEY_umask		228
-#define KEY_undef		229
-#define KEY_unless		230
-#define KEY_unlink		231
-#define KEY_unpack		232
-#define KEY_unshift		233
-#define KEY_untie		234
-#define KEY_until		235
-#define KEY_use			236
-#define KEY_utime		237
-#define KEY_values		238
-#define KEY_vec			239
-#define KEY_wait		240
-#define KEY_waitpid		241
-#define KEY_wantarray		242
-#define KEY_warn		243
-#define KEY_while		244
-#define KEY_write		245
-#define KEY_x			246
-#define KEY_xor			247
-#define KEY_y			248
+#define KEY_our			142
+#define KEY_pack		143
+#define KEY_package		144
+#define KEY_pipe		145
+#define KEY_pop			146
+#define KEY_pos			147
+#define KEY_print		148
+#define KEY_printf		149
+#define KEY_prototype		150
+#define KEY_push		151
+#define KEY_q			152
+#define KEY_qq			153
+#define KEY_qr			154
+#define KEY_quotemeta		155
+#define KEY_qw			156
+#define KEY_qx			157
+#define KEY_rand		158
+#define KEY_read		159
+#define KEY_readdir		160
+#define KEY_readline		161
+#define KEY_readlink		162
+#define KEY_readpipe		163
+#define KEY_recv		164
+#define KEY_redo		165
+#define KEY_ref			166
+#define KEY_rename		167
+#define KEY_require		168
+#define KEY_reset		169
+#define KEY_return		170
+#define KEY_reverse		171
+#define KEY_rewinddir		172
+#define KEY_rindex		173
+#define KEY_rmdir		174
+#define KEY_s			175
+#define KEY_scalar		176
+#define KEY_seek		177
+#define KEY_seekdir		178
+#define KEY_select		179
+#define KEY_semctl		180
+#define KEY_semget		181
+#define KEY_semop		182
+#define KEY_send		183
+#define KEY_setgrent		184
+#define KEY_sethostent		185
+#define KEY_setnetent		186
+#define KEY_setpgrp		187
+#define KEY_setpriority		188
+#define KEY_setprotoent		189
+#define KEY_setpwent		190
+#define KEY_setservent		191
+#define KEY_setsockopt		192
+#define KEY_shift		193
+#define KEY_shmctl		194
+#define KEY_shmget		195
+#define KEY_shmread		196
+#define KEY_shmwrite		197
+#define KEY_shutdown		198
+#define KEY_sin			199
+#define KEY_sleep		200
+#define KEY_socket		201
+#define KEY_socketpair		202
+#define KEY_sort		203
+#define KEY_splice		204
+#define KEY_split		205
+#define KEY_sprintf		206
+#define KEY_sqrt		207
+#define KEY_srand		208
+#define KEY_stat		209
+#define KEY_study		210
+#define KEY_sub			211
+#define KEY_substr		212
+#define KEY_symlink		213
+#define KEY_syscall		214
+#define KEY_sysopen		215
+#define KEY_sysread		216
+#define KEY_sysseek		217
+#define KEY_system		218
+#define KEY_syswrite		219
+#define KEY_tell		220
+#define KEY_telldir		221
+#define KEY_tie			222
+#define KEY_tied		223
+#define KEY_time		224
+#define KEY_times		225
+#define KEY_tr			226
+#define KEY_truncate		227
+#define KEY_uc			228
+#define KEY_ucfirst		229
+#define KEY_umask		230
+#define KEY_undef		231
+#define KEY_unless		232
+#define KEY_unlink		233
+#define KEY_unpack		234
+#define KEY_unshift		235
+#define KEY_untie		236
+#define KEY_until		237
+#define KEY_use			238
+#define KEY_utime		239
+#define KEY_values		240
+#define KEY_vec			241
+#define KEY_wait		242
+#define KEY_waitpid		243
+#define KEY_wantarray		244
+#define KEY_warn		245
+#define KEY_while		246
+#define KEY_write		247
+#define KEY_x			248
+#define KEY_xor			249
+#define KEY_y			250
