@@ -244,8 +244,8 @@ my %items = ();			# associative array used to find the location
 my $Is83;                       # is dos with short filenames (8.3)
 
 sub init_globals {
-$dircache = "pod2html.d~~";
-$itemcache = "pod2html.i~~";
+$dircache = "pod2htmd$cache_ext";
+$itemcache = "pod2htmi$cache_ext";
 
 @begin_stack = ();		# begin/end stack
 
@@ -1506,7 +1506,8 @@ sub process_L {
 	$link = "#" . htmlify(0,$section);
 	$linktext = $section unless defined($linktext);
     } elsif ( $page =~ /::/ ) {
-	$linktext  = ($section ? "$section" : "$page");
+	$linktext  = ($section ? "$section" : "$page")
+	    unless defined($linktext);
 	$page =~ s,::,/,g;
 	# Search page cache for an entry keyed under the html page name,
 	# then look to see what directory that page might be in.  NOTE:
