@@ -277,6 +277,7 @@ return <<'END';
 2133	2139
 2160	2183
 2336	237a
+2395	
 249c	24e9
 3005	3007
 3021	3029
@@ -301,11 +302,6 @@ return <<'END';
 3400	4db5
 4e00	9fa5
 a000	a48c
-a490	a4a1
-a4a4	a4b3
-a4b5	a4c0
-a4c2	a4c4
-a4c6	
 ac00	d7a3
 d800	db7f
 db80	dbff
