@@ -53,7 +53,8 @@ return <<'END';
 2214	22f1
 2300	2335
 237b	
-237d	239a
+237d	2394
+2396	239a
 2400	2426
 2440	244a
 2500	2595
@@ -84,6 +85,11 @@ return <<'END';
 303e	303f
 309b	309c
 30fb	
+a490	a4a1
+a4a4	a4b3
+a4b5	a4c0
+a4c2	a4c4
+a4c6	
 fd3e	fd3f
 fe30	fe44
 fe49	fe4f
