@@ -1,6 +1,8 @@
 return <<'END';
 0000	001f
 007f	009f
+070f	
+180b	180e
 200c	200f
 202a	202e
 206a	206f
@@ -9,4 +11,5 @@ db80	dbff
 dc00	dfff
 e000	f8ff
 feff	
+fff9	fffb
 END
