@@ -11,11 +11,20 @@ return <<'END';
 207a	207c
 208a	208c
 2190	2194
+219a	219b
+21a0	
+21a3	
+21a6	
+21ae	
+21ce	21cf
 21d2	
 21d4	
 2200	22f1
 2308	230b
 2320	2321
+25b7	
+25c1	
+266f	
 fb29	
 fe62	
 fe64	fe66
