@@ -30,7 +30,13 @@ return <<'END';
 212e	
 2132	
 213a	
-2195	21d1
+2195	2199
+219c	219f
+21a1	21a2
+21a4	21a5
+21a7	21ad
+21af	21cd
+21d0	21d1
 21d3	
 21d5	21f3
 2300	2307
@@ -42,9 +48,12 @@ return <<'END';
 2440	244a
 249c	24e9
 2500	2595
-25a0	25f7
+25a0	25b6
+25b8	25c0
+25c2	25f7
 2600	2613
-2619	2671
+2619	266e
+2670	2671
 2701	2704
 2706	2709
 270c	2727
@@ -79,6 +88,11 @@ return <<'END';
 3300	3376
 337b	33dd
 33e0	33fe
+a490	a4a1
+a4a4	a4b3
+a4b5	a4c0
+a4c2	a4c4
+a4c6	
 ffe4	
 ffe8	
 ffed	ffee
