@@ -8,10 +8,12 @@
 
 
 #include "EXTERN.h"
+#define PERL_IN_MINIPERLMAIN_C
 #include "perl.h"
 
-static void xs_init _((void));
+static void xs_init (pTHX);
 static PerlInterpreter *my_perl;
+static struct perl_thread *thr;		/* XXXXXX testing */
 
 #if defined (__MINT__) || defined (atarist)
 /* The Atari operating system doesn't have a dynamic stack.  The
@@ -26,18 +28,18 @@ main(int argc, char **argv, char **env)
 
 #ifdef PERL_GLOBAL_STRUCT
 #define PERLVAR(var,type) /**/
+#define PERLVARA(var,type) /**/
 #define PERLVARI(var,type,init) PL_Vars.var = init;
 #define PERLVARIC(var,type,init) PL_Vars.var = init;
 #include "perlvars.h"
 #undef PERLVAR
+#undef PERLVARA
 #undef PERLVARI
 #undef PERLVARIC
 #endif
 
     PERL_SYS_INIT(&argc,&argv);
 
-    perl_init_i18nl10n(1);
-
     if (!PL_do_undump) {
 	my_perl = perl_alloc();
 	if (!my_perl)
@@ -65,7 +67,7 @@ main(int argc, char **argv, char **env)
 /* Do not delete this line--writemain depends on it */
 
 static void
-xs_init(void)
+xs_init(pTHX)
 {
   dXSUB_SYS;
 }
