@@ -5,7 +5,23 @@
 
 #define PERL_REVISION	5		/* age */
 #define PERL_VERSION	5		/* epoch */
-#define PERL_SUBVERSION	61		/* generation */
+#define PERL_SUBVERSION	62		/* generation */
+
+/* Compatibility across versions:  MakeMaker will install add-on
+   modules in a directory with the PERL_APIVERSION version number.  
+   Normally this should not change across maintenance releases.
+   perl.c:incpush() and lib/lib.pm will automatically search older 
+   directories across major versions back to to PERL_XS_APIVERSION
+   for XS modules and back to PERL_PM_APIVERSION for pure PERL modules.
+   (Since the versioned directory layout didn't start until 5.005,
+   that's the earliest these can go back.
+
+   See INSTALL for how this works.
+*/
+#define PERL_APIVERSION 5.00562		/* Adjust manually as needed.  */
+
+#define __PATCHLEVEL_H_INCLUDED__
+#endif
 
 /*
 	local_patches -- list of locally applied less-than-subversion patches.
@@ -42,6 +58,7 @@
 	This will prevent patch from choking if someone has previously
 	applied different patches than you.
  */
+#if !defined(PERL_PATCHLEVEL_H_IMPLICIT) && !defined(LOCAL_PATCH_COUNT)
 static	char	*local_patches[] = {
 	NULL
 	,NULL
@@ -51,13 +68,8 @@ static	char	*local_patches[] = {
 #  define	LOCAL_PATCH_COUNT	\
 	(sizeof(local_patches)/sizeof(local_patches[0])-2)
 
-#  define __PATCHLEVEL_H_INCLUDED__
-#endif
-
 /* the old terms of reference, add them only when explicitly included */
-#if !defined(PERL_PATCHLEVEL_H_IMPLICIT) && !defined(PATCHLEVEL)
 #define PATCHLEVEL		PERL_VERSION
 #undef  SUBVERSION		/* OS/390 has a SUBVERSION in a system header */
 #define SUBVERSION		PERL_SUBVERSION
 #endif
-
