@@ -618,7 +618,7 @@
 /* HAS_SHMAT_PROTOTYPE:
  *	This symbol, if defined, indicates that the sys/shm.h includes
  *	a prototype for shmat().  Otherwise, it is up to the program to
- *	guess one.  Shmat_t shmat _((int, Shmat_t, int)) is a good guess,
+ *	guess one.  Shmat_t shmat (int, Shmat_t, int) is a good guess,
  *	but not always right so it should be emitted by the program only
  *	when HAS_SHMAT_PROTOTYPE is not defined to avoid conflicting defs.
  */
